"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DOMRenderer = void 0;
var tslib_1 = require("tslib");
var web_utility_1 = require("web-utility");
var VDOM_1 = require("./VDOM");
var DOMRenderer = /** @class */ (function () {
    function DOMRenderer() {
        var _this = this;
        this.eventPattern = /^on[A-Z]/;
        this.ariaPattern = /^aira[A-Z]/;
        this.treeCache = new WeakMap();
        this.keyOf = function (_a, index) {
            var key = _a.key, text = _a.text, props = _a.props, selector = _a.selector;
            return (key === null || key === void 0 ? void 0 : key.toString()) || (props === null || props === void 0 ? void 0 : props.id) || text || (selector && selector + index);
        };
        this.vNodeOf = function (list, key) {
            return list.find(function (vNode, index) { return "".concat(_this.keyOf(vNode, index)) === String(key); });
        };
        this.propsKeyOf = function (key) {
            return key.startsWith('aria-')
                ? (0, web_utility_1.toCamelCase)(key)
                : _this.eventPattern.test(key)
                    ? key.toLowerCase()
                    : key;
        };
    }
    DOMRenderer.prototype.updateProps = function (node, oldProps, newProps, onDelete, onAdd) {
        if (oldProps === void 0) { oldProps = {}; }
        if (newProps === void 0) { newProps = {}; }
        var group = (0, web_utility_1.diffKeys)(Object.keys(oldProps), Object.keys(newProps)).group;
        for (var _i = 0, _a = group[web_utility_1.DiffStatus.Old] || []; _i < _a.length; _i++) {
            var key = _a[_i][0];
            onDelete === null || onDelete === void 0 ? void 0 : onDelete(node, key);
        }
        for (var _b = 0, _c = tslib_1.__spreadArray(tslib_1.__spreadArray([], (group[web_utility_1.DiffStatus.Same] || []), true), (group[web_utility_1.DiffStatus.New] || []), true); _b < _c.length; _b++) {
            var key = _c[_b][0];
            if (oldProps[key] !== newProps[key])
                if (onAdd instanceof Function)
                    onAdd(node, key, newProps[key]);
                else
                    Reflect.set(node, key, newProps[key]);
        }
    };
    DOMRenderer.prototype.createNode = function (vNode) {
        var _a;
        if (vNode.text)
            return (vNode.node = document.createTextNode(vNode.text));
        vNode.node = vNode.tagName
            ? document.createElement(vNode.tagName, { is: vNode.is })
            : document.createDocumentFragment();
        var node = this.patch({ tagName: vNode.tagName, node: vNode.node }, vNode).node;
        if (node)
            (_a = vNode.ref) === null || _a === void 0 ? void 0 : _a.call(vNode, node);
        return node;
    };
    DOMRenderer.prototype.deleteNode = function (_a) {
        var node = _a.node, children = _a.children;
        if (node instanceof DocumentFragment)
            children === null || children === void 0 ? void 0 : children.forEach(this.deleteNode);
        else
            node === null || node === void 0 ? void 0 : node.remove();
    };
    DOMRenderer.prototype.updateChildren = function (node, oldList, newList) {
        var _this = this;
        var _a = (0, web_utility_1.diffKeys)(oldList.map(this.keyOf), newList.map(this.keyOf)), map = _a.map, group = _a.group;
        for (var _i = 0, _b = group[web_utility_1.DiffStatus.Old] || []; _i < _b.length; _i++) {
            var key = _b[_i][0];
            this.deleteNode(this.vNodeOf(oldList, key));
        }
        var newNodes = newList.map(function (vNode, index) {
            var key = _this.keyOf(vNode, index);
            if (map[key] !== web_utility_1.DiffStatus.Same)
                return _this.createNode(vNode);
            var oldVNode = _this.vNodeOf(oldList, key);
            return vNode.text != null
                ? (vNode.node = oldVNode.node)
                : _this.patch(oldVNode, vNode).node;
        });
        node.append.apply(node, newNodes);
    };
    DOMRenderer.prototype.patch = function (oldVNode, newVNode) {
        var _this = this;
        var tagName = oldVNode.tagName;
        var isXML = (0, web_utility_1.templateOf)(tagName) && (0, web_utility_1.elementTypeOf)(tagName) === 'xml';
        this.updateProps(oldVNode.node, oldVNode.props, newVNode.props, function (node, key) {
            return _this.eventPattern.test(key)
                ? (node[key.toLowerCase()] = null)
                : node.removeAttribute(_this.ariaPattern.test(key)
                    ? (0, web_utility_1.toHyphenCase)(key)
                    : VDOM_1.VNode.propsMap[key] || key);
        }, function (node, key, value) {
            // @ts-ignore
            if (isXML || key.includes('-') || (0, web_utility_1.isDOMReadOnly)(tagName, key))
                node.setAttribute(key, value);
            else
                node[_this.propsKeyOf(key)] = value;
        });
        this.updateProps(oldVNode.node.style, oldVNode.style, newVNode.style, function (node, key) { return node.removeProperty(key); });
        this.updateChildren(oldVNode.node, oldVNode.children || [], newVNode.children || []);
        newVNode.node = oldVNode.node;
        return newVNode;
    };
    DOMRenderer.prototype.render = function (vNode, node) {
        if (node === void 0) { node = document.body; }
        var root = this.treeCache.get(node) || VDOM_1.VNode.fromDOM(node);
        root = this.patch(root, tslib_1.__assign(tslib_1.__assign({}, root), { children: [vNode] }));
        this.treeCache.set(node, root);
        return root;
    };
    DOMRenderer.prototype.renderToStaticMarkup = function (tree) {
        var body = document.implementation.createHTMLDocument().body;
        this.render(tree, body);
        return body.innerHTML;
    };
    return DOMRenderer;
}());
exports.DOMRenderer = DOMRenderer;
